/* checklib.inc */

/*
 * Standard allocate and I/O functions with checking of errors and
 * with print error messages on error output.
 */

#include "sysdeps.h"

#include <stddef.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

typedef struct {
	FILE *file;
	char *name;
} extFILE;

/* chybov hlky jako konstanty */
#define NumErrMsg     "Chyba . %d pi "
#define ErrMsg        "Chyba pi "
#define TextMsg       "%s\n"

#define FileOpenMsg   "otevrn souboru \"%s\":\n"
#define FileCloseMsg  "zavrn souboru \"%s\":\n"
#define FileReadMsg   "ten ze souboru \"%s\":\n"

#define AllocMsg      "alokaci pamti (poadovno %d byt).\n"



/* alokace pamti s kontrolou a vpisem chyb */
bool mallocOK(void **point, size_t len) {
	if (!(*point=malloc(len))) {
		fprintf(stderr, ErrMsg AllocMsg, len);
		return FALSE;
	}
	return TRUE;
}


/* alokace datov struktury pro soubor */
bool fileAllocOK(extFILE **extFile, FILE *f, char *name) {
	if (!(*extFile=malloc(sizeof(extFILE)))) {
		fprintf(stderr, ErrMsg AllocMsg, sizeof(extFILE));
		return FALSE;
	}
	(*extFile)->file=f;
	(*extFile)->name=name;
	return TRUE;
}


/* zaven souboru s vypsnm ppadn chyby */
bool closeFile(extFILE *f) {
	if (fclose(f->file)!=0) {
		fprintf(stderr, NumErrMsg FileCloseMsg ":\n" TextMsg, errno, f->name, strerror(errno));
		return FALSE;
	}
	return TRUE;
}


/* oteven souboru s vypsnm ppadn chyby */
bool openFile(extFILE **f, char *name, const char *typeOpen) {
	if (!(*f=malloc(sizeof(extFILE)))) {
		fprintf(stderr, ErrMsg FileOpenMsg ErrMsg AllocMsg "\n", name, sizeof(extFILE));
		return FALSE;
	}
	if (((*f)->file=fopen(name, typeOpen))==NULL) {
		fprintf(stderr, NumErrMsg FileOpenMsg TextMsg, errno, name, strerror(errno));
		return FALSE;
	}
	(*f)->name=name;
	return TRUE;
}


/* pete znak */
/* nvratov hodnoty:
	O.K. ...........znak
	konec souboru...EOF, err=FALSE
	chyba ten.....EOF, err=TRUE
 */
int readChar(extFILE *f, bool *err) {
	int z;

	if ((z=fgetc(f->file))==EOF)
		if (ferror(f->file)) {
			fprintf(stderr, NumErrMsg FileReadMsg TextMsg, errno, f->name, strerror(errno));
			*err=TRUE;
		} else *err=FALSE;
	else *err=FALSE;
	return z;
}


