#define traceTEX2HTM

#define maxLenOfForm 256

#define readErrMsg       "Chyba . %d pi ten ze souboru \"%s\":\n%s\n"

#define escapeStr1       "<script language=\"latex\">"
#define escapeStr2       "</script>"

#define latexHead "\
\%definice\n\
\%\\def\\tg{\\mathop{\\rm tg}\\nolimits}\n\
\\def\\operator@font{\\mathgroup\\symoperators}\n\
\\def\\tg{\\mathop{\\operator@font tg}\\nolimits}\n\
\%\\def\\tg{\\qopname\\relax o{sin}}\n\
\%\\def\\tg{\\mathop{tg}\\nolimits}\n\
\n\
\\documentclass{minimal}\n\
\\begin{document}\n\
"
#define latexTail "\n\
\\end{document}\n\
"

#define help_message "\
Pouit:\n\
  %s <strnka.html.in> [<strnka.html>]\n\
\n\
  Pi neuveden clovho argumentu jde vsledek na standardn vstup.\n\
\n\
\n\
Pevod vce strnek:\n\
  %s <strnka1.html.in> <strnka1.html> [<druh> <tet> ...]\n\
\n\
  Pi pouit pevodu vce strnek je nutno uvdet ke kad i cl.\n\
"

#define tmpname "vzorec%d"

#include <errno.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "checklib.h"



/* ---------- globln promnn ---------- */
int  formCount=0;



/* retval: "z" je oddlova */
bool isWhiteChar(char z) {
	return z==' ' || z=='\n' || z=='\t';
}


/* retval: "z" je psmeno */
bool isCharacter(char z) {
	return (z>='a' && z<='z') || (z>='A' && z<='Z');
}


/* If je pozice nastavena na dan tag "tag", nastav se pozice za nj (TRUE)
   Jinak se petne nedefinovan poet znak do "cell" (a vrt FALSE).

  poznmka: nutno zajistit, aby pro "cell" bylo msto >= ne pro "tag"
*/
bool isTag(extFILE *f, const char *tag, char *cell, bool *isEnd, bool *err) {
	int i=0; /* cell length */
	int j=0; /* pointer to char in required tag */
	int len; /* length of required tag */

	int z;   /* readed character */

	len=strlen(tag);
/* smyka pete jen nkolik znak (maximln 'strlen(tag)') */
	do {
		z=readChar(f, err);
		if (z!=EOF) cell[i++]=z;
		j++;
	} while (z!=EOF && z==tag[j-1] && j<len);
/* konec */
	/* Jestlie volajcho zajm monost konce souboru (isEnd!=NULL),
	 * dod se mu to. */
	if (isEnd) *isEnd=z==EOF;
	/* zakonen etzce */
	cell[i]=0;
/* vrac, jestli byl nalezen dan tag */
	return j==len;
}


/* vrac:
 * a) err=FALSE
 * 	TRUE....v "cell" je sekvence mezi "tag1" a "tag2"
 * 	FALSE...v "cell" je pvodn sekvence (nebo EOF)
 * b) err=TRUE
 * 	chyba ten
 */
bool tag(extFILE *f, const char *tag1, const char *tag2, char *cell, bool *err) {
	bool end;

	int i=0;
	int j=0;

	int z;

	if (isTag(f, tag1, cell, &end, err) && !end) {
	/* nael sekvenci, pete to mezi tim */
		i=0;
		j=0;
		do {
			z=readChar(f, err);
			if (z!=EOF) {
				cell[i++]=z;
			}
			if (z!=tag2[j++]) j=0;
		} while (z!=EOF && j<strlen(escapeStr2));
		i-=j;
		cell[i++]=0;
		return TRUE;  /* nael sekvenci */
	} else
		return FALSE; /* nenael sekvenci, vrac originl */
}


/* posune ukazatel v souboru za zadan "tag" */
bool seekBehindTag(extFILE *f, const char *tag) {
	char *cell;
	bool is;
	bool end, err;

	cell=malloc(strlen(tag)+1);
	do {
		is=isTag(f, tag, cell, &end, &err);
	} while(!is && !end && !err);
	free(cell);
	return is && !end && !err;
}


/* prava etzce:
 *   odsekn okrajovch blch znak, nahrazen vech blch znak
 *   pouze mezerami
 */
char *stripWhites(char *depr) {
	char *cut;
	int i, j, k;

/* przdn nebo neplatn etzec ==> konec, vrac NULL */
	if (!depr) return NULL;
	if (strlen(depr)==0) return NULL;

/* nalezen hranice etzce */
	/* zatek etzce */
	i=0;
	while (depr[i]!=0 && isWhiteChar(depr[i])) i++;
	/* konec etzce */
	if ((j=strlen(depr))!=0)
		while (isWhiteChar(depr[j]) && j>i) j--;

/* alokuje a zkopruje nov etzec */
	/* alokace */
	if (!mallocOK((void **)&cut, j-i+1+1)) return NULL;
	/* koprovn */
	k=0;
	while (i<=j) {
		cut[k++]=isWhiteChar(depr[i]) ? ' ':depr[i];
		i++;
	}
	cut[i]=0;

	return cut;
}


/* Vstupuje text latexu, zape se do novho .tex souboru i s hlavikami,
 * do .html souboru zape odkazy na obrzky vzorc. */
bool convertAndPrint(FILE *fo, char *latex) {
	char name[10];
	char nameext[14];
	char *stripped;
	extFILE *tmpf;

/* prava etzce */
	if (!(stripped=stripWhites(latex))) return TRUE;

/* generovn jmna souboru */
#ifndef nosnprintf
	snprintf(name, 10, tmpname, formCount);
#else
	sprintf(name, tmpname, formCount);
#endif
	formCount++;
/* vytvoen doasnho souboru latexu */
	strcpy(nameext, name);
	strcat(nameext, ".tex");
	if (!openFile(&tmpf, nameext, "wt")) {
		free(stripped);
		return FALSE;
	}
	fprintf(tmpf->file, "%s%s%s", latexHead, stripped, latexTail);
	free(stripped);
	if (!closeFile(tmpf)) return FALSE;
/* zapsn obrzku do .html */
	strcpy(nameext, name);
	strcat(nameext, ".gif");
	fprintf(fo, "<img src=\"%s\" alt=\"%s\">\n", nameext, latex);

	return TRUE;
}


/* zpracovn souboru "name" - zape do "fo" */
bool doingFile(char *name, extFILE *fo) {
	extFILE *fi;
	char cell[maxLenOfForm];
	bool err;

/* oteven vstupnho souboru */
	if (!openFile(&fi, name, "rt")) return FALSE;

/* zpracovn */
	do {
		if (tag(fi, escapeStr1, escapeStr2, cell, &err)) {
		/* v "cell" je latex */
			if (!convertAndPrint(fo->file, cell)) {
				closeFile(fi);
				closeFile(fo);
				return FALSE;
			}
		} else {
		/* v "cell" je originl strnka */
			if (cell[0]!=0) fprintf(fo->file, "%s", cell);
		}
	} while (cell[0]!=0);

/* konec */
	/* zaven soubor */
	return closeFile(fi);
}


int main(int argn, char *argv[]) {
	extFILE *fo;
	int param;

/* pprava - inicializace */
	/* nutno alespo jeden argument */
	if (argn<1+1) {
		fprintf(stderr, help_message, argv[0], argv[0]);
		return 1;
	}

/* vstupn soubor (diskov i stdout) */
	if (argn==1+1) {
	/* jeden vstup, standardn vstup */
		if (!fileAllocOK(&fo, stdout, "<stdout>")) return 3;
		doingFile(argv[1], fo);
	} else {
	/* zadno vce strnek ke zpracovn,
	   vdy dvojice "<vstupn soubor> <vstupn soubor>" */
		if ((argn-1) % 2!=0) {
			fprintf(stderr, "Poet argument by ml bt sud (vstupn a vstupn soubor).\n");
			return 1;
		}
		/* zpracovn vech strnek zadanch jako argument */
		for (param=1; param<argn-1; param++) {
			if (!openFile(&fo, argv[param << 1], "wt")) return 2;
			doingFile(argv[(param << 1)-1], fo);
		}
	}

/* zaven vstupnho souboru */
	if (fo->file!=stdout) closeFile(fo);

	return 0;
}
